@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gio/gio.h>
 #include <gio/gdesktopappinfo.h>
 #include <packagekit-glib2/packagekit.h>
 
@@ -41,6 +42,7 @@
 #include "gs-shell-search-provider.h"
 #include "gs-offline-updates.h"
 #include "gs-folders.h"
+#include "gs-utils.h"
 
 
 struct _GsApplication {
@@ -91,7 +93,8 @@ download_updates_setting_changed (GSettings     *settings,
                                   const gchar   *key,
                                   GsApplication *app)
 {
-	if (g_settings_get_boolean (settings, key)) {
+	if (!gs_updates_are_managed () &&
+	    g_settings_get_boolean (settings, key)) {
 		g_debug ("Enabling update monitor");
 		app->update_monitor = gs_update_monitor_new (app);
 	} else {
@@ -100,6 +103,27 @@ download_updates_setting_changed (GSettings     *settings,
 	}
 }
 
+static void
+on_permission_changed (GPermission *permission,
+                       GParamSpec  *pspec,
+                       gpointer     data)
+{
+	GsApplication *app = data;
+
+	if (app->settings)
+		download_updates_setting_changed (app->settings, "download-updates", app);
+}
+
+static void
+gs_application_monitor_permission (GsApplication *app)
+{
+	GPermission *permission;
+
+	permission = gs_offline_updates_permission_get ();
+	g_signal_connect (permission, "notify",
+			  G_CALLBACK (on_permission_changed), app);
+}
+
 static void
 gs_application_monitor_updates (GsApplication *app)
 {
@@ -115,7 +139,6 @@ network_changed_cb (GNetworkMonitor *monitor,
 		    gboolean available,
 		    GsApplication *app)
 {
-	g_debug ("*** Network status change: %s", available ? "online" : "offline");
 	gs_plugin_loader_set_network_status (app->plugin_loader, available);
 }
 
@@ -221,6 +244,20 @@ gs_application_initialize_ui (GsApplication *app)
 				  G_CALLBACK (gtk_window_present), gs_shell_get_window (app->shell));
 }
 
+static void
+initialize_ui_and_present_window (GsApplication *app)
+{
+	GList *windows;
+	GtkWindow *window;
+
+	gs_application_initialize_ui (app);
+	windows = gtk_application_get_windows (GTK_APPLICATION (app));
+	if (windows) {
+		window = windows->data;
+		gtk_window_present (window);
+	}
+}
+
 static void
 sources_activated (GSimpleAction *action,
 		   GVariant      *parameter,
@@ -315,15 +352,8 @@ set_mode_activated (GSimpleAction *action,
 {
 	GsApplication *app = GS_APPLICATION (data);
 	const gchar *mode;
-	GList *windows;
-	GtkWindow *window = NULL;
 
-	gs_application_initialize_ui (app);
-	windows = gtk_application_get_windows (GTK_APPLICATION (app));
-	if (windows) {
-		window = windows->data;
-		gtk_window_present (window);
-	}
+	initialize_ui_and_present_window (app);
 
 	mode = g_variant_get_string (parameter, NULL);
 	if (g_strcmp0 (mode, "updates") == 0) {
@@ -347,15 +377,8 @@ search_activated (GSimpleAction *action,
 {
 	GsApplication *app = GS_APPLICATION (data);
 	const gchar *search;
-	GList *windows;
-	GtkWindow *window = NULL;
 
-	gs_application_initialize_ui (app);
-	windows = gtk_application_get_windows (GTK_APPLICATION (app));
-	if (windows) {
-		window = windows->data;
-		gtk_window_present (window);
-	}
+	initialize_ui_and_present_window (app);
 
 	search = g_variant_get_string (parameter, NULL);
 	gs_shell_show_search (app->shell, search);
@@ -367,17 +390,10 @@ details_activated (GSimpleAction *action,
 		   gpointer       data)
 {
 	GsApplication *app = GS_APPLICATION (data);
-	GList *windows;
-	GtkWindow *window = NULL;
 	const gchar *id;
 	const gchar *search;
 
-	windows = gtk_application_get_windows (GTK_APPLICATION (app));
-	if (windows) {
-		window = windows->data;
-		gtk_window_present (window);
-	}
-	gs_application_initialize_ui (app);
+	initialize_ui_and_present_window (app);
 
 	g_variant_get (parameter, "(&s&s)", &id, &search);
 	if (search != NULL && search[0] != '\0')
@@ -447,7 +463,12 @@ show_offline_updates_error (GSimpleAction *action,
 			    GVariant      *parameter,
 			    gpointer       data)
 {
-	gs_offline_updates_show_error ();
+	GsApplication *app = GS_APPLICATION (data);
+
+	initialize_ui_and_present_window (app);
+
+	gs_shell_set_mode (app->shell, GS_SHELL_MODE_UPDATES);
+	gs_offline_updates_show_error (app->shell);
 }
 
 static GActionEntry actions[] = {
@@ -479,6 +500,7 @@ gs_application_startup (GApplication *application)
 	GS_APPLICATION (application)->proxy_settings = gs_proxy_settings_new ();
 	GS_APPLICATION (application)->dbus_helper = gs_dbus_helper_new ();
 	GS_APPLICATION (application)->settings = g_settings_new ("org.gnome.software");
+	gs_application_monitor_permission (GS_APPLICATION (application));
 	gs_application_monitor_updates (GS_APPLICATION (application));
 	gs_application_provide_search (GS_APPLICATION (application));
 	gs_application_monitor_network (GS_APPLICATION (application));
