@@ -33,6 +33,7 @@
 #include "gs-markdown.h"
 #include "gs-update-dialog.h"
 #include "gs-update-list.h"
+#include "gs-application.h"
 
 #include <gdesktop-enums.h>
 #include <langinfo.h>
@@ -45,6 +46,7 @@ typedef enum {
 	GS_SHELL_UPDATES_STATE_ACTION_REFRESH_HAS_UPDATES,
 	GS_SHELL_UPDATES_STATE_ACTION_GET_UPDATES,
 	GS_SHELL_UPDATES_STATE_NO_UPDATES,
+	GS_SHELL_UPDATES_STATE_MANAGED,
 	GS_SHELL_UPDATES_STATE_HAS_UPDATES,
 	GS_SHELL_UPDATES_STATE_FAILED,
 	GS_SHELL_UPDATES_STATE_LAST,
@@ -236,6 +238,7 @@ gs_shell_updates_update_ui_state (GsShellUpdates *shell_updates)
 		break;
 	case GS_SHELL_UPDATES_STATE_ACTION_REFRESH_HAS_UPDATES:
 	case GS_SHELL_UPDATES_STATE_NO_UPDATES:
+	case GS_SHELL_UPDATES_STATE_MANAGED:
 	case GS_SHELL_UPDATES_STATE_HAS_UPDATES:
 	case GS_SHELL_UPDATES_STATE_FAILED:
 		gs_stop_spinner (GTK_SPINNER (priv->spinner_updates));
@@ -268,6 +271,7 @@ gs_shell_updates_update_ui_state (GsShellUpdates *shell_updates)
 		break;
 	case GS_SHELL_UPDATES_STATE_ACTION_REFRESH_HAS_UPDATES:
 	case GS_SHELL_UPDATES_STATE_NO_UPDATES:
+	case GS_SHELL_UPDATES_STATE_MANAGED:
 	case GS_SHELL_UPDATES_STATE_HAS_UPDATES:
 	case GS_SHELL_UPDATES_STATE_FAILED:
 		break;
@@ -286,6 +290,7 @@ gs_shell_updates_update_ui_state (GsShellUpdates *shell_updates)
 	case GS_SHELL_UPDATES_STATE_ACTION_REFRESH_NO_UPDATES:
 	case GS_SHELL_UPDATES_STATE_ACTION_GET_UPDATES:
 	case GS_SHELL_UPDATES_STATE_NO_UPDATES:
+	case GS_SHELL_UPDATES_STATE_MANAGED:
 	case GS_SHELL_UPDATES_STATE_HAS_UPDATES:
 	case GS_SHELL_UPDATES_STATE_STARTUP:
 	case GS_SHELL_UPDATES_STATE_FAILED:
@@ -309,6 +314,7 @@ gs_shell_updates_update_ui_state (GsShellUpdates *shell_updates)
 		break;
 	case GS_SHELL_UPDATES_STATE_ACTION_GET_UPDATES:
 	case GS_SHELL_UPDATES_STATE_STARTUP:
+	case GS_SHELL_UPDATES_STATE_MANAGED:
 		widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_refresh"));
 		gtk_widget_hide (widget);
 		break;
@@ -342,6 +348,7 @@ gs_shell_updates_update_ui_state (GsShellUpdates *shell_updates)
 	case GS_SHELL_UPDATES_STATE_ACTION_REFRESH_NO_UPDATES:
 	case GS_SHELL_UPDATES_STATE_ACTION_GET_UPDATES:
 	case GS_SHELL_UPDATES_STATE_NO_UPDATES:
+	case GS_SHELL_UPDATES_STATE_MANAGED:
 	case GS_SHELL_UPDATES_STATE_FAILED:
 		gtk_widget_hide (widget);
 		break;
@@ -383,6 +390,9 @@ gs_shell_updates_update_ui_state (GsShellUpdates *shell_updates)
 	case GS_SHELL_UPDATES_STATE_ACTION_REFRESH_HAS_UPDATES:
 		gtk_stack_set_visible_child_name (GTK_STACK (priv->stack_updates), "view");
 		break;
+	case GS_SHELL_UPDATES_STATE_MANAGED:
+		gtk_stack_set_visible_child_name (GTK_STACK (priv->stack_updates), "managed");
+		break;
 	case GS_SHELL_UPDATES_STATE_FAILED:
 		gtk_stack_set_visible_child_name (GTK_STACK (priv->stack_updates), "failed");
 		break;
@@ -416,6 +426,8 @@ gs_shell_updates_set_state (GsShellUpdates *shell_updates,
 			    GsShellUpdatesState state)
 {
 	shell_updates->priv->state = state;
+	if (gs_updates_are_managed ())
+		shell_updates->priv->state = GS_SHELL_UPDATES_STATE_MANAGED;
 	gs_shell_updates_update_ui_state (shell_updates);
 }
 
@@ -454,7 +466,7 @@ gs_shell_updates_get_updates_cb (GsPluginLoader *plugin_loader,
 	}
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_updates_counter"));
-	if (list != NULL) {
+	if (list != NULL && !gs_updates_are_managed ()) {
 		gchar *text;
 		text = g_strdup_printf ("%d", g_list_length (list));
 		gtk_label_set_label (GTK_LABEL (widget), text);
@@ -901,6 +913,32 @@ gs_shell_updates_status_changed_cb (GsPluginLoader *plugin_loader,
 	gs_shell_updates_update_ui_state (shell_updates);
 }
 
+static void
+on_permission_changed (GPermission *permission,
+                       GParamSpec  *pspec,
+                       gpointer     data)
+{
+        GsShellUpdates *shell_updates = data;
+        GsShellUpdatesPrivate *priv = shell_updates->priv;
+
+	if (gs_updates_are_managed()) {
+		gs_shell_updates_set_state (shell_updates, GS_SHELL_UPDATES_STATE_MANAGED);
+	}
+	else if (priv->state == GS_SHELL_UPDATES_STATE_MANAGED) {
+		gs_shell_updates_set_state (shell_updates, GS_SHELL_UPDATES_STATE_NO_UPDATES);
+	}
+}
+
+static void
+gs_shell_updates_monitor_permission (GsShellUpdates *shell_updates)
+{
+        GPermission *permission;
+
+        permission = gs_offline_updates_permission_get ();
+        g_signal_connect (permission, "notify",
+                          G_CALLBACK (on_permission_changed), shell_updates);
+}
+
 void
 gs_shell_updates_setup (GsShellUpdates *shell_updates,
 			GsShell *shell,
@@ -944,6 +982,8 @@ gs_shell_updates_setup (GsShellUpdates *shell_updates,
 			  G_CALLBACK (gs_shell_updates_button_network_settings_cb),
 			  shell_updates);
 
+	gs_shell_updates_monitor_permission (shell_updates);
+
 	g_signal_connect (priv->control, "notify::network-state",
 			  G_CALLBACK (gs_shell_updates_notify_network_state_cb),
 			  shell_updates);
